#include <BoolField.h>
#include <DegreeField.h>
#include <Field.h>
#include <Form.h>
#include <IntField.h>
#include <LCD.h>
#include <ListField.h>
#include <TextField.h>
#include <TimeField.h>
#include <GetIntField.h>
#include <TimerOne.h>
#define stepPin 1
#define dirPin 2
#define ledPin 13
#define CLOCKWISE     HIGH
#define ANTICLOCKWISE LOW
#define MINUTES_IN_CIRCLE 1296000
#define STEPS_PER_REV 180000 // = 2000 * 90
#define TIMER_DELAY 40
LCD lcd;
int dir = CLOCKWISE;  // Clockwise
int state = LOW;

volatile long steps = 0;
long divisions;
long stepsPerDiv;
int ctr = 0;

const char item_GoDiv[] PROGMEM = "UP or DWN Starts";
//const char item_Cheese[] PROGMEM = "Cheese";
//const char item_Pumpkin[] PROGMEM = "Pumpkin";
 ListItem const DivideMenu[] PROGMEM = {
    item_GoDiv,
   // item_Cheese,
   // item_Pumpkin,
    0
 };
Form mainForm(lcd);
TextField welcomeField(mainForm, "RotTable Control", "Left/Right/Up/Dn");
IntField divisionField(mainForm, "Divisions", 1, 100, 1, 3, " ");
DegreeField degreeField(mainForm, "Division Angle", 360, DEGREEFIELD_READ_ONLY);
ListField goDivideField(mainForm, "   <Go Divide>", DivideMenu);
void godivide(void);

void setup() 
{
  // Initialize the digital pin as an output.
  // Pin 13 has an LED connected on most Arduino boards
  pinMode(ledPin, OUTPUT);    
  pinMode(stepPin, OUTPUT);
  pinMode(dirPin, OUTPUT);

  Timer1.initialize(TIMER_DELAY); // set a timer of length in microseconds (or 0.1 sec - or 10Hz)
  Timer1.attachInterrupt( timerIsr ); // attach the service routine here
    // Status LED initially on.
   digitalWrite(ledPin, HIGH);
  
  // Enable the screen saver, which will automatically blank the screen after 10 seconds.
  // The screen will wake up again when a button is pressed or lcd.display() is called.
  lcd.enableScreenSaver();
  
  // Show the main form for the first time.
  mainForm.show();

}
 
void loop()
{
  int i;
  divisions = divisionField.value();
  stepsPerDiv = (long)STEPS_PER_REV / divisions;
   degreeField.setValue((long)(((float)MINUTES_IN_CIRCLE / (float)STEPS_PER_REV) * (float)stepsPerDiv));
  // Dispatch button events to the main form.
  int event = lcd.getButton();
  if (mainForm.dispatch(event) == FORM_CHANGED) {
    if (mainForm.isCurrent(divisionField)) {
        divisions = (long)divisionField.value();
        stepsPerDiv = (long)STEPS_PER_REV / divisions;
        degreeField.setValue((long)(((float)MINUTES_IN_CIRCLE / (float)STEPS_PER_REV) * (float)stepsPerDiv));
    }  
    if (mainForm.isCurrent(goDivideField)) {
      godivide();
    }
  }
}
 
void printDivideVals(int iDiv)
{
   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print("Div: ");
   lcd.print(iDiv);
   lcd.print(" of ");
   lcd.print(divisionField.value());
   degreeField.printDegrees();
   lcd.print(" / Div");  
}

void godivide(void)
{
   long thisDiv = 1; 
   int event = LCD_BUTTON_NONE;
   printDivideVals(thisDiv);
   while(event != LCD_BUTTON_SELECT){
      event = lcd.getButton();
      switch(event){
        case LCD_BUTTON_LEFT:
          digitalWrite( dirPin, ANTICLOCKWISE );
          thisDiv--;
          if(thisDiv < 1)
            thisDiv = divisions;          
          while(steps){
            ;
          }
          steps = stepsPerDiv * 2;
          printDivideVals(thisDiv);
          break;
        case LCD_BUTTON_RIGHT:
          digitalWrite( dirPin, CLOCKWISE );
          thisDiv++;
          if(thisDiv > divisions)
            thisDiv = 1;          
          while(steps){
            ;
          }
          steps = stepsPerDiv * 2; 
          printDivideVals(thisDiv);
          break;
    } 
  }
}
/// --------------------------
/// Custom ISR Timer Routine
/// --------------------------
void timerIsr()
{
    // Toggle LED
  if(steps){
      digitalWrite( ledPin, digitalRead( ledPin ) ^ 1 );
      state = state ^ 1;
      digitalWrite( stepPin, state );
      steps--;
  }
}
